#! /bin/bash -eu

if (( $# < 3 ))
then
  echo "usage: $0 <input.bbm> <output.bbm> <output.asm>" >&2
  exit 2
fi
INPUT="$1"
OUTPUT="$2"
OUTPUT_ASM="$3"

SCRIPTDIR="$(dirname "$0")"
MASK="$SCRIPTDIR/mask.bbm"

BC="BlinkenConv"

MAP="0/0,3 1/1,3 2/0,4 3/1,4
     4/0,0 5/1,0 6/0,1 7/1,1
     8/3,3 9/4,3 10/3,4 11/4,4
     12/3,0 13/4,0 14/3,1 15/4,1
     16/6,3 17/7,3 18/6,4 19/7,4
     20/6,0 21/7,0 22/6,1 23/7,1
     24/9,3 25/10,3 26/9,4 27/10,4
     28/12,3 29/13,3 30/12,4 31/13,4"

TMP="$(mktemp -d "$SCRIPTDIR/tmp.XXXXXXXX")"

DURATION="$("$BC" -i "$INPUT" -f \
               | sed -n 's/^duration: *\([0-9]\+\) ms$/\1/p')"

PIXELS="$TMP/pixels.bbm"
"$BC" -t 32x1-1/16 black "$DURATION" -o "$PIXELS"
for M in $MAP
do
  SRC="${M#*/}"
  DEST="${M%/*},0"
  "$BC" -i "$PIXELS" -C "$INPUT" "$SRC" 1x1 "$DEST" -o "$PIXELS"
done

"$BC" -i "mask.bbm" -dur "$DURATION" -o "$OUTPUT"
for M in $MAP
do
  SRC="${M%/*},0"
  DEST="${M#*/}"
  "$BC" -i "$OUTPUT" -C "$PIXELS" "$SRC" 1x1 "$DEST" -o "$OUTPUT"
done

PIXELS_BMM="$TMP/pixels.bmm"
"$BC" -i "$PIXELS" -r 32x1-1/16 -o "$PIXELS_BMM"

sed -n 's/^\(\(0x[0-9A-F]\+ *\)\+\|@[0-9]\+\)$/&/p' "$PIXELS_BMM" \
  | sed 's/0x\([0-9A-F]\)[^ ]* \+0x\([0-9A-F]\)[^ ]*/0x\1\2,/g' \
  | tr '\n@' ' \n' \
  | while read DUR DATA
    do
      if test -n "$DUR"
      then
        MS4="$(((DUR + 4) / 4))"
        while (( MS4 > 255 ))
        do
          echo "0, 255, $DATA"
          MS4="$((MS4 - 255))"
        done
        if (( MS4 > 0 ))
        then
          echo "        .DB     0, $(($MS4)), $DATA"
        fi
      fi
    done \
  > "$OUTPUT_ASM"

rm -rf "$TMP"

